/* Copyright (c) 2009, Code Aurora Forum. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Code Aurora Forum nor
 *       the names of its contributors may be used to endorse or promote
 *       products derived from this software without specific prior written
 *       permission.
 *
 * Alternatively, provided that this notice is retained in full, this software
 * may be relicensed by the recipient under the terms of the GNU General Public
 * License version 2 ("GPL") and only version 2, in which case the provisions of
 * the GPL apply INSTEAD OF those given above.  If the recipient relicenses the
 * software under the GPL, then the identification text in the MODULE_LICENSE
 * macro must be changed to reflect "GPLv2" instead of "Dual BSD/GPL".  Once a
 * recipient changes the license terms to the GPL, subsequent recipients shall
 * not relicense under alternate licensing terms, including the BSD or dual
 * BSD/GPL terms.  In addition, the following license statement immediately
 * below and between the words START and END shall also then apply when this
 * software is relicensed under the GPL:
 *
 * START
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License version 2 and only version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * END
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <linux/kernel.h>
#include <linux/slab.h>

#include <mach/pmic.h>
#include <mach/qdsp6/msm8k_cad_ioctl.h>
#include <mach/qdsp6/msm8k_ard_adiei.h>
#include <mach/qdsp6/msm8k_adie_codec_rpc.h>
#include <mach/qdsp6/msm8k_adie_codec_dev.h>
#include <mach/qdsp6/msm8k_cad_devices.h>
#include <mach/qdsp6/msm8k_ard_clk.h>

#include <mach/pm_log.h>

#include <linux/moduleparam.h>
#include <mach/austin_hwid.h>


#include <linux/module.h>

#include <linux/delay.h>


#include <mach/gpio.h>
#if defined(CONFIG_MACH_EVT0) ||defined(CONFIG_MACH_EVB)
#define AUDIO_SPK_DRV_SHTDWN_N	108
#endif
#define AUDIO_HPSHDN_SHTDWN_N	109

#define AUDIO_HEADSET_MIC_SHTDWN_N	17




int g_max_spk_gain 		= SPKR_GAIN_PLUS12DB;
int g_lineout_spk_gain 	= SPKR_GAIN_PLUS12DB;
static int g_ac_5v_status	=0;


int g_is_pmic_mute = 0;

module_param(g_max_spk_gain, int, 0644);
module_param(g_lineout_spk_gain, int, 0644);
module_param(g_ac_5v_status, int, 0644);
module_param(g_is_pmic_mute, int, 0644);




struct adie_state_struct_type	adie_state;
int pmic_is_stereo;
static int pmic_initialized;

static u32 adie_spkr_mono_ref1_cnt;
static u32 adie_spkr_stereo_ref1_cnt;

static u32 adie_spkr_mono_ref2_cnt;
static u32 adie_spkr_stereo_ref2_cnt;

#if 1
#define D(fmt, args...) printk(KERN_INFO "adie: " fmt, ##args)
#else
#define D(fmt, args...) do {} while (0)
#endif


void adie_update_AC_5V_status(int ac_status)
{
	g_ac_5v_status = ac_status;
	D("adie_update_AC_5V_status: g_ac_5v_status = %d", g_ac_5v_status );
}
EXPORT_SYMBOL(adie_update_AC_5V_status);


u32 get_path_id(u32 dev_id)
{
	u32 adie_path_id = 0;

	switch (dev_id) {
	case CAD_HW_DEVICE_ID_HANDSET_MIC:
		adie_path_id = DAL_ADIE_CODEC_HANDSET_TX;
		break;
			
	case CAD_HW_DEVICE_ID_HANDSET_SPKR:
	case CAD_HW_DEVICE_ID_HANDSET_SPKR_HAC:	
		adie_path_id = DAL_ADIE_CODEC_HANDSET_RX;
		break;
			
	case CAD_HW_DEVICE_ID_HEADSET_MIC:
		adie_path_id = DAL_ADIE_CODEC_HEADSET_MONO_TX;
		break;
	case CAD_HW_DEVICE_ID_HEADSET_SPKR_MONO:
		adie_path_id = DAL_ADIE_CODEC_HEADSET_MONO_RX;
		break;
	case CAD_HW_DEVICE_ID_HEADSET_SPKR_STEREO:
		adie_path_id = DAL_ADIE_CODEC_HEADSET_STEREO_RX;
		break;
	case CAD_HW_DEVICE_ID_SPKR_PHONE_MIC:
		adie_path_id = DAL_ADIE_CODEC_SPEAKER_TX;
		break;
	case CAD_HW_DEVICE_ID_SPKR_PHONE_MONO:
		adie_path_id = DAL_ADIE_CODEC_SPEAKER_RX;
		break;
	
	case CAD_HW_DEVICE_ID_SPEAKER_SPKR_STEREO_LINE_OUT:
	case CAD_HW_DEVICE_ID_SPKR_PHONE_STEREO:
		adie_path_id = DAL_ADIE_CODEC_SPEAKER_STEREO_RX;
		break;
	
	case CAD_HW_DEVICE_ID_TTY_HEADSET_MIC:
		adie_path_id = DAL_ADIE_CODEC_TTY_HEADSET_TX;
		break;
	case CAD_HW_DEVICE_ID_TTY_HEADSET_SPKR:
		adie_path_id = DAL_ADIE_CODEC_TTY_HEADSET_RX;
		break;
	case CAD_HW_DEVICE_ID_HEADSET_MONO_PLUS_SPKR_STEREO_RX:
		adie_path_id = DAL_ADIE_CODEC_SPKR_STEREO_HDPH_MONO_RX;
		break;
	case CAD_HW_DEVICE_ID_HEADSET_MONO_PLUS_SPKR_MONO_RX:
		adie_path_id = DAL_ADIE_CODEC_SPKR_MONO_HDPH_MONO_RX;
		break;
	case CAD_HW_DEVICE_ID_HEADSET_STEREO_PLUS_SPKR_MONO_RX:
		adie_path_id = DAL_ADIE_CODEC_SPKR_MONO_HDPH_STEREO_RX;
		break;
	case CAD_HW_DEVICE_ID_HEADSET_STEREO_PLUS_SPKR_STEREO_RX:
		adie_path_id = DAL_ADIE_CODEC_SPKR_STEREO_HDPH_STEREO_RX;
		break;
	case CAD_HW_DEVICE_ID_HEADSET_SPKR_STEREO_LB:
		adie_path_id = DAL_ADIE_CODEC_AUXPGA_HDPH_STEREO_LB;
		break;
	case CAD_HW_DEVICE_ID_HEADSET_SPKR_MONO_LB:
		adie_path_id = DAL_ADIE_CODEC_AUXPGA_HDPH_MONO_LB;
		break;
	case CAD_HW_DEVICE_ID_SPEAKER_SPKR_STEREO_LB:
		adie_path_id = DAL_ADIE_CODEC_AUXPGA_LINEOUT_STEREO_LB;
		break;
	case CAD_HW_DEVICE_ID_SPEAKER_SPKR_MONO_LB:
		adie_path_id = DAL_ADIE_CODEC_AUXPGA_LINEOUT_MONO_LB;
		break;
	
	case CAD_HW_DEVICE_ID_VOICE_RECOGNITION_MIC:
		
		if (HSD_HEADSET == g_headset_type)
			adie_path_id = DAL_ADIE_CODEC_HEADSET_MONO_TX;
		else
			adie_path_id = DAL_ADIE_CODEC_SPEAKER_TX;
		break;
		
	case CAD_HW_DEVICE_ID_BT_SCO_MIC:
	case CAD_HW_DEVICE_ID_BT_SCO_SPKR:
	case CAD_HW_DEVICE_ID_BT_A2DP_SPKR:
	case CAD_HW_DEVICE_ID_BT_A2DP_TX:
	default:
		pr_err("ARD ADIE Paths not supported for dev_id %d\n", dev_id);
		break;
	}
	return adie_path_id;
}

u32 get_path_type(u32 cad_dev_type)
{
	u32 adie_dev_type;

	switch (cad_dev_type) {
	case CAD_RX_DEVICE:
		adie_dev_type = ADIE_CODEC_RX;
		break;
	case CAD_TX_DEVICE:
		adie_dev_type = ADIE_CODEC_TX;
		break;
	default:
		adie_dev_type = ADIE_CODEC_LB;
		break;
	}
	D("ARD ADIE DEV TYPE = %d\n", adie_dev_type);

	return adie_dev_type;
}

void pmic_init(void)
{
	s32	dal_rc;
	u8	dev_type;
	u32	stereo;

	dal_rc = CAD_RES_SUCCESS;
	dev_type = 0;

	if (pmic_spkr_is_stereo_en(&stereo))
		pmic_is_stereo = 0;
	else
		pmic_is_stereo = 1;

	D("pmic stereo out val = 0x%08x, pmic_initialized = %d\n", stereo, pmic_initialized);

	
	if (pmic_is_stereo) {
		pmic_spkr_set_gain(LEFT_SPKR, SPKR_GAIN_PLUS12DB);
		pmic_spkr_set_gain(RIGHT_SPKR, SPKR_GAIN_PLUS12DB);
	} else{
		pmic_set_speaker_gain(SPKR_GAIN_PLUS12DB);
	}
	
	pmic_speaker_cmd(SPKR_OFF);
	
	pmic_spkr_en_left_chan(0);
	pmic_spkr_en_right_chan(0);
	
	pmic_spkr_en_hpf(0);
	
	
	PM_LOG_EVENT(PM_LOG_OFF, PM_LOG_AUDIO_SPK);


	pmic_mic_set_volt(MIC_VOLT_2_00V);
	
	pmic_spkr_set_mux_hpf_corner_freq(SPKR_FREQ_1_39KHZ);

	pmic_initialized = 1;
}

s32 adie_init(void)
{
	s32	dal_rc;
	u8	dev_type;

	dal_rc = CAD_RES_SUCCESS;
	dev_type = 0;

	dal_rc = daldevice_attach(DALDEVICEID_ADIE_CODEC, ADIE_CODEC_PORT_NAME,
			ADIE_CODEC_CPU, &adie_state.adie_handle);
	if (CAD_RES_SUCCESS != dal_rc)
		pr_err("ARD: ADIE Device Attach failed\n");

	adie_state.adie_opened = ADIE_FALSE;

	
	for (dev_type = 0; dev_type < MAX_ADIE_PATH_TYPES; dev_type++) {
		adie_state.adie_path_type[dev_type].enabled = ADIE_FALSE;
		adie_state.adie_path_type[dev_type].enable_request = ADIE_FALSE;
		adie_state.adie_path_type[dev_type].state = ADIE_STATE_RESET;
	}

	return dal_rc;
}

s32 adie_dinit(void)
{
	s32	dal_rc;
	u8	dev_type;

	dal_rc = CAD_RES_SUCCESS;
	dev_type = 0;

	dal_rc = daldevice_detach(adie_state.adie_handle);
	if (CAD_RES_SUCCESS != dal_rc)
		pr_err("ARD: ADIE Device Detach failed\n");

	for (dev_type = 0; dev_type < MAX_ADIE_PATH_TYPES; dev_type++) {
		adie_state.adie_path_type[dev_type].enabled = ADIE_FALSE;
		adie_state.adie_path_type[dev_type].enable_request = ADIE_FALSE;
		adie_state.adie_path_type[dev_type].state = ADIE_STATE_RESET;
	}

	pmic_speaker_cmd(SPKR_DISABLE);
	
	PM_LOG_EVENT(PM_LOG_OFF, PM_LOG_AUDIO_SPK);

	adie_state.adie_opened = ADIE_FALSE;

	return dal_rc;
}

s32 adie_open(u32 dev_type)
{
	s32 cad_rc, dal_rc;

	cad_rc = dal_rc = CAD_RES_SUCCESS;

	if (!pmic_initialized)
		pmic_init();

	if (adie_state.adie_opened != ADIE_TRUE) {
		dal_rc = daldevice_open(adie_state.adie_handle, 0);

		if (dal_rc != CAD_RES_SUCCESS) {
			pr_err("ARD ADIE DAL Open failed, dev_type = %d\n",
				dev_type);
			cad_rc = CAD_RES_FAILURE;
		} else {
			adie_state.adie_opened = ADIE_TRUE;
			D("ARD ADIE DRIVER OPENED\n");
		}
	}
	return cad_rc;
}


s32 adie_close(u32 dev_type)
{
	s32 rc;

	rc = CAD_RES_SUCCESS;

	if (adie_state.adie_opened == ADIE_TRUE) {
		if (((adie_state.adie_path_type[CAD_RX_DEVICE]).enabled
					== ADIE_TRUE) ||
			((adie_state.adie_path_type[CAD_TX_DEVICE]).enabled
					== ADIE_TRUE) ||
			((adie_state.adie_path_type[CAD_AUXPGA_DEVICE]).enabled
					== ADIE_TRUE)) {
			
		} else {
			rc = daldevice_close(adie_state.adie_handle);
			adie_state.adie_opened = ADIE_FALSE;
		}
	}

	return rc;
}

s32 adie_enable(u32 dev_type, u32 dev_id)
{
	s32 rc = CAD_RES_SUCCESS;

	if (adie_state.adie_path_type[dev_type].enabled != ADIE_TRUE) {
		adie_state.adie_path_type[dev_type].enable_request = ADIE_TRUE;
		rc = adie_state_control(dev_type, dev_id);
		if (rc != CAD_RES_SUCCESS)
			pr_err("ADIE STATE M/C FAILED, dev_id = %d\n", dev_id);
		else
			D("ADIE ENABLED - PATH %d\n", dev_type);
	} else
		rc = CAD_RES_FAILURE;

	return rc;
}

s32 adie_disable(u32 dev_type, u32 dev_id)
{
	s32		rc = CAD_RES_SUCCESS;

	if (adie_state.adie_path_type[dev_type].enabled != ADIE_FALSE) {
		adie_state.adie_path_type[dev_type].enabled = ADIE_FALSE;

		rc = adie_state_control(dev_type, dev_id);
		if (rc != CAD_RES_SUCCESS)
			pr_err("ADIE STATE M/C FAILED, dev_id = %d\n", dev_id);
		else
			D("ADIE DISABLED\n");
	} else {
		pr_err("ADIE ALREADY DISABLED, dev_id = %d\n", dev_id);
		rc = CAD_RES_FAILURE;
	}

	return rc;
}

u32 adie_state_control(u32 dev_type, u32 dev_id)
{
	s32				cad_res;
	enum adie_state_ret_enum_type	rc;

	cad_res = CAD_RES_SUCCESS;
	rc = ADIE_STATE_RC_SUCCESS;

	do {
		D("ARD ADIE dev_type %d, adie state %d\n",
			dev_type, adie_state.adie_path_type[dev_type].state);

		switch (adie_state.adie_path_type[dev_type].state) {
		case ADIE_STATE_RESET:
			rc = adie_state_reset(dev_type, dev_id);
			break;
		case ADIE_STATE_DIGITAL_ACTIVE:
			rc = adie_state_digital_active(dev_type, dev_id);
			break;
		case ADIE_STATE_DIGITAL_ANALOG_ACTIVE:
			rc = adie_state_digital_analog_active(dev_type, dev_id);
			break;
		default:
			break;
		}
	} while (rc == ADIE_STATE_RC_CONTINUE);

	if (rc == ADIE_STATE_RC_FAILURE)
		cad_res = CAD_RES_FAILURE;

	return cad_res;
}

enum adie_state_ret_enum_type adie_state_reset(u32 dev_type, u32 dev_id)
{
	enum adie_state_ret_enum_type	rc;
	s32				dal_rc;
	enum adie_codec_path_type_enum	path_type;
	u32				path_id;
	u32				freq_plan;

	rc = ADIE_STATE_RC_SUCCESS;
	dal_rc = CAD_RES_SUCCESS;
	if (adie_state.adie_path_type[dev_type].enable_request != ADIE_TRUE) {
		rc = ADIE_STATE_RC_SUCCESS;
		adie_state.adie_path_type[dev_type].enabled = ADIE_FALSE;
		goto done;
	}

	path_id = get_path_id(dev_id);
	path_type = get_path_type(dev_type);

	if (path_type == ADIE_CODEC_RX)
		freq_plan = 48000;
	else {
		if (g_clk_info.tx_clk_freq > 16000)
			freq_plan = 48000;
		else if (g_clk_info.tx_clk_freq > 8000)
			freq_plan = 16000;
		else
			freq_plan = 8000;
	}

	
	dal_rc = rpc_adie_codec_set_path(adie_state.adie_handle, path_id,
			path_type);

	if (dal_rc != CAD_RES_SUCCESS) {
		pr_err("ARD ADIE Set Path failed for dev_type %d\n", dev_type);
		rc = ADIE_STATE_RC_FAILURE;
		goto done;
	}

	
	dal_rc = rpc_adie_codec_set_path_freq_plan(adie_state.adie_handle,
			path_type, freq_plan);

	if (dal_rc != CAD_RES_SUCCESS) {
		pr_err("ARD ADIE Set Path Freq Plan failed for dev_type %d\n",
			dev_type);
		rc = ADIE_STATE_RC_FAILURE;
		goto done;
	}

	
	dal_rc = rpc_adie_codec_proceed_to_stage(adie_state.adie_handle,
			path_type, ADIE_CODEC_DIGITAL_READY);

	if (dal_rc != CAD_RES_SUCCESS) {
		pr_err("ARD ADIE Proceed to Stage failed for dev_type %d\n",
			dev_type);
		rc = ADIE_STATE_RC_FAILURE;
		goto done;
	}

	adie_state.adie_path_type[dev_type].state = ADIE_STATE_DIGITAL_ACTIVE;
	rc = ADIE_STATE_RC_CONTINUE;
done:
	return rc;
}

static int is_speaker_mono(u32 dev_id)
{
	if ((dev_id == CAD_HW_DEVICE_ID_SPKR_PHONE_MONO) ||
		(dev_id == CAD_HW_DEVICE_ID_SPEAKER_SPKR_MONO_LB) ||
		(dev_id == CAD_HW_DEVICE_ID_HEADSET_MONO_PLUS_SPKR_MONO_RX) ||
		(dev_id == CAD_HW_DEVICE_ID_HEADSET_STEREO_PLUS_SPKR_MONO_RX))
		return 1;
	else
		return 0;
}

static int is_speaker_stereo(u32 dev_id)
{
	
	if ((CAD_HW_DEVICE_ID_SPKR_PHONE_STEREO == dev_id) ||
		(CAD_HW_DEVICE_ID_SPEAKER_SPKR_STEREO_LB == dev_id) ||
		(CAD_HW_DEVICE_ID_HEADSET_MONO_PLUS_SPKR_STEREO_RX
				== dev_id) ||
		(CAD_HW_DEVICE_ID_HEADSET_STEREO_PLUS_SPKR_STEREO_RX
				== dev_id)||
		(CAD_HW_DEVICE_ID_SPEAKER_SPKR_STEREO_LINE_OUT
				== dev_id))
		return 1;
	else
		return 0;
	
}

enum adie_state_ret_enum_type adie_state_digital_active(u32 dev_type,
							u32 dev_id)
{
	enum adie_state_ret_enum_type	rc;
	enum adie_codec_path_type_enum	path_type;
	s32				dal_rc;

	rc = ADIE_STATE_RC_SUCCESS;
	dal_rc = CAD_RES_SUCCESS;

	path_type = get_path_type(dev_type);
	if (adie_state.adie_path_type[dev_type].enable_request == ADIE_TRUE) {

		
		
		if (path_type == ADIE_CODEC_TX)
		{
			if ((dev_id == CAD_HW_DEVICE_ID_HANDSET_MIC) ||
			    (dev_id == CAD_HW_DEVICE_ID_SPKR_PHONE_MIC) ||
			    ( (dev_id ==CAD_HW_DEVICE_ID_VOICE_RECOGNITION_MIC) && (HSD_HEADSET != g_headset_type) )) 
			{

				pmic_mic_en(ON_CMD);
				D("Turn on mic en");
				
				PM_LOG_EVENT(PM_LOG_ON, PM_LOG_AUTIO_MIC);
			}
			else if ((dev_id == CAD_HW_DEVICE_ID_TTY_HEADSET_MIC) && (HSD_EAR_PHONE == g_headset_type ))
			{
				
				D("Turn off mic en on TTY_MIC and turn on headset mic shutdown pin\n");
				pmic_mic_en(OFF_CMD);
				if ((system_rev > EVT1_4G_4G ) && (system_rev != HWID_UNKNOWN) )
				{
					
					gpio_set_value(AUDIO_HEADSET_MIC_SHTDWN_N, 0);
					
					PM_LOG_EVENT(PM_LOG_ON, PM_LOG_AUTIO_MIC);
				}
			}
			else
			{
				D("adie_state_digital_active: ADIE_CODEC_TX: Do not thing!!");
			}
		}
		else if ((path_type == ADIE_CODEC_RX) ||
			(path_type == ADIE_CODEC_LB)) {
			struct spkr_config_mode scm;
			memset(&scm, 0, sizeof(struct spkr_config_mode));

			if (is_speaker_mono(dev_id)) {
				if (!adie_spkr_mono_ref1_cnt)
				{
					if (pmic_is_stereo)
					{
						scm.is_right_chan_en = 0;
						scm.is_left_chan_en = 1;
						scm.is_stereo_en = 1;
						scm.is_hpf_en = 1;

						pmic_spkr_en_mute(
							LEFT_SPKR, 0);
						pmic_spkr_en_mute(
							RIGHT_SPKR, 0);

						pmic_set_spkr_configuration(
								&scm);

						pmic_spkr_en(LEFT_SPKR, 1);
						pmic_spkr_en(RIGHT_SPKR, 0);
					} else {
						pmic_speaker_cmd(SPKR_MUTE_ON);
						pmic_speaker_cmd(SPKR_ENABLE);
					}
#if defined(CONFIG_MACH_EVT0) ||defined(CONFIG_MACH_EVB)
						gpio_set_value(AUDIO_SPK_DRV_SHTDWN_N, 1);
#endif
				}

				
				adie_spkr_mono_ref1_cnt++;
			}
			else if (is_speaker_stereo(dev_id))
			{
				if (!adie_spkr_stereo_ref1_cnt) {
					if (pmic_is_stereo)
					{
						
						int speaker_gain = 0;

						scm.is_right_chan_en = 1;
						scm.is_left_chan_en = 1;
						scm.is_stereo_en = 1;
						scm.is_hpf_en = 1;
						scm.is_right_left_chan_added =1;

						pmic_spkr_en_mute(
							LEFT_SPKR, 0);
						pmic_spkr_en_mute(
							RIGHT_SPKR, 0);

						pmic_set_spkr_configuration(
								&scm);
						
						if(1 == g_ac_5v_status )
						{
							speaker_gain = g_lineout_spk_gain;
						}
						else
						{
							speaker_gain = g_max_spk_gain;
						}
						printk("Reset speaker gain before enable \n");

						pmic_spkr_set_gain(LEFT_SPKR, speaker_gain);
						pmic_spkr_set_gain(RIGHT_SPKR, speaker_gain);

						pmic_spkr_en(LEFT_SPKR, 1);
						pmic_spkr_en(RIGHT_SPKR, 1);
						
						pmic_spkr_en_left_chan(1);
						pmic_spkr_en_right_chan(1);
						
						pmic_spkr_en_hpf(1);
						
#if defined(CONFIG_MACH_EVT0) ||defined(CONFIG_MACH_EVB)
						gpio_set_value(AUDIO_SPK_DRV_SHTDWN_N, 1);
#endif
						
						PM_LOG_EVENT(PM_LOG_ON, PM_LOG_AUDIO_SPK);
					}
					else
					{
						pmic_speaker_cmd(SPKR_MUTE_ON);
						pmic_speaker_cmd(SPKR_ENABLE);
					}
				}

				
				adie_spkr_stereo_ref1_cnt++;
			}
			else
			{
				pr_err("bad mode: not stereo or mono\n");
			}
		}

		
		dal_rc = rpc_adie_codec_proceed_to_stage(adie_state.adie_handle,
				path_type, ADIE_CODEC_DIGITAL_ANALOG_READY);

		if (dal_rc != CAD_RES_SUCCESS) {
			pr_err("ARD ADIE Proceed to Stage failed,"
				" dev_type %d\n", dev_type);
			rc = ADIE_STATE_RC_FAILURE;
			goto done;
		}

		adie_state.adie_path_type[dev_type].state =
			ADIE_STATE_DIGITAL_ANALOG_ACTIVE;
		adie_state.adie_path_type[dev_type].enabled = ADIE_TRUE;
		adie_state.adie_path_type[dev_type].enable_request = ADIE_FALSE;
		rc = ADIE_STATE_RC_CONTINUE;
	}
	else
	{

		if (path_type == ADIE_CODEC_TX)
		{

			if ((dev_id == CAD_HW_DEVICE_ID_HANDSET_MIC) ||
			    (dev_id == CAD_HW_DEVICE_ID_SPKR_PHONE_MIC)  ||
			    ( (dev_id ==CAD_HW_DEVICE_ID_VOICE_RECOGNITION_MIC) && (HSD_HEADSET != g_headset_type) )
			    )
			{

				pmic_mic_en(OFF_CMD);
				D("Turn off mic en");
				
				PM_LOG_EVENT(PM_LOG_OFF, PM_LOG_AUTIO_MIC);
			}
			else if ( (dev_id == CAD_HW_DEVICE_ID_TTY_HEADSET_MIC)  && (HSD_EAR_PHONE == g_headset_type) )
			{

				if ((system_rev > EVT1_4G_4G ) && (system_rev != HWID_UNKNOWN) )
				{
					
					gpio_set_value(AUDIO_HEADSET_MIC_SHTDWN_N, 1);
					
					PM_LOG_EVENT(PM_LOG_OFF, PM_LOG_AUTIO_MIC);
				}
			}
			else
			{
				D("Do not thing!! dev_id %d", dev_id);
			}
		}
		else if ((path_type == ADIE_CODEC_RX) ||
				(path_type == ADIE_CODEC_LB))
		{
			if (is_speaker_mono(dev_id))
			{
				
				adie_spkr_mono_ref1_cnt--;

				
				if (!adie_spkr_mono_ref1_cnt)
				{
					if (pmic_is_stereo)
					{
						pmic_spkr_en(LEFT_SPKR, 0);
						pmic_spkr_en(RIGHT_SPKR, 0);				
					}
					else
					{
						
						pmic_set_speaker_gain(g_max_spk_gain);
						pmic_speaker_cmd(SPKR_DISABLE);
					}
				}
#if defined(CONFIG_MACH_EVT0) ||defined(CONFIG_MACH_EVB)
				gpio_set_value(AUDIO_SPK_DRV_SHTDWN_N, 0);
#endif
				
				PM_LOG_EVENT(PM_LOG_OFF, PM_LOG_AUDIO_SPK);
			}
			else if (is_speaker_stereo(dev_id))
			{

				
				adie_spkr_stereo_ref1_cnt--;

				
				if (!adie_spkr_stereo_ref1_cnt)
				{
					if (pmic_is_stereo)
					{
						pmic_spkr_en(LEFT_SPKR, 0);
						pmic_spkr_en(RIGHT_SPKR, 0);
						
						pmic_spkr_en_left_chan(0);
						pmic_spkr_en_right_chan(0);	
						
						pmic_spkr_en_hpf(0);						
					}
					else
					{
						pmic_speaker_cmd(SPKR_DISABLE);
					}
			}
#if defined(CONFIG_MACH_EVT0) ||defined(CONFIG_MACH_EVB)
				gpio_set_value(AUDIO_SPK_DRV_SHTDWN_N, 0);
#endif
				
				PM_LOG_EVENT(PM_LOG_OFF, PM_LOG_AUDIO_SPK);
			}
		}




		
		dal_rc = rpc_adie_codec_proceed_to_stage(adie_state.adie_handle,
				path_type, ADIE_CODEC_DIGITAL_OFF);

		if (dal_rc != CAD_RES_SUCCESS) {
			pr_err("ARD ADIE Proceed to Stage failed,"
				" dev_type %d\n", dev_type);
			rc = ADIE_STATE_RC_FAILURE;
			goto done;
		}

		adie_state.adie_path_type[dev_type].state = ADIE_STATE_RESET;
		rc = ADIE_STATE_RC_CONTINUE;
	}
done:
	return rc;
}

enum adie_state_ret_enum_type adie_state_digital_analog_active(u32 dev_type,
							       u32 dev_id)
{
	s32				dal_rc;
	enum adie_state_ret_enum_type   rc;
	enum adie_codec_path_type_enum	path_type;

	dal_rc = CAD_RES_SUCCESS;
	rc = ADIE_STATE_RC_SUCCESS;
	path_type = get_path_type(dev_type);

	if (adie_state.adie_path_type[dev_type].enabled == ADIE_TRUE) {
		

		if ((path_type == ADIE_CODEC_RX) ||
			(path_type == ADIE_CODEC_LB))
		{
			if (is_speaker_mono(dev_id))
			{
				
				if (!adie_spkr_mono_ref2_cnt)
				{
					if (pmic_is_stereo)
					{
						pmic_spkr_en_mute(LEFT_SPKR, 1);
					}
					else
					{
						pmic_speaker_cmd(SPKR_MUTE_OFF);
					}
				}

				
				adie_spkr_mono_ref2_cnt++;

			}
			else if (is_speaker_stereo(dev_id))
			{

				
				if (!adie_spkr_stereo_ref2_cnt)
				{
					if (pmic_is_stereo)
					{
						
						if ((g_is_pmic_mute == 0 ) || (dev_id != CAD_HW_DEVICE_ID_SPEAKER_SPKR_STEREO_LINE_OUT))
						{
							D("adie_state_digital_analog_active: unmute pmic speaker dev_id = %d",dev_id);
							pmic_spkr_en_mute(LEFT_SPKR, 1);
							pmic_spkr_en_mute(RIGHT_SPKR, 1);
						}
						
						
						if ( (dev_id == CAD_HW_DEVICE_ID_HEADSET_MONO_PLUS_SPKR_STEREO_RX) ||
							(dev_id == CAD_HW_DEVICE_ID_HEADSET_STEREO_PLUS_SPKR_STEREO_RX))
						{
							gpio_set_value(AUDIO_HPSHDN_SHTDWN_N, 1);
							
							PM_LOG_EVENT(PM_LOG_ON, PM_LOG_AUDIO_HS);
							msleep(50);
						}
						
					} else
					{
						pmic_speaker_cmd(SPKR_MUTE_OFF);
					}
				}

				
				adie_spkr_stereo_ref2_cnt++;
			}
			else if ((dev_id == CAD_HW_DEVICE_ID_HEADSET_SPKR_MONO) ||
					(dev_id == CAD_HW_DEVICE_ID_HEADSET_SPKR_STEREO) ||
					(dev_id == CAD_HW_DEVICE_ID_HEADSET_SPKR_STEREO_LB) ||
					(dev_id == CAD_HW_DEVICE_ID_HEADSET_SPKR_MONO_LB) ||
					(dev_id == CAD_HW_DEVICE_ID_TTY_HEADSET_SPKR))
			{
				gpio_set_value(AUDIO_HPSHDN_SHTDWN_N, 1);
				
				PM_LOG_EVENT(PM_LOG_ON, PM_LOG_AUDIO_HS);
				msleep(50);
			}
		}
		else
		{
			D("ARD ADIE Loopback Device\n");
		}

	}
	else
	{

		if ((path_type == ADIE_CODEC_RX) ||
			(path_type == ADIE_CODEC_LB))
		{
			if (is_speaker_mono(dev_id))
			{

				
				adie_spkr_mono_ref2_cnt--;

				if (!adie_spkr_mono_ref2_cnt)
				{
					if (pmic_is_stereo)
					{
						pmic_spkr_en_mute(LEFT_SPKR, 0);
					}
					else
					{
						pmic_speaker_cmd(SPKR_MUTE_ON);
					}
				}
			}
			else if (is_speaker_stereo(dev_id))
			{

				
				adie_spkr_stereo_ref2_cnt--;

				if (!adie_spkr_stereo_ref2_cnt)
				{
					if (pmic_is_stereo)
					{
						pmic_spkr_en_mute(LEFT_SPKR, 0);
						pmic_spkr_en_mute(RIGHT_SPKR, 0);
						
						if ( (dev_id == CAD_HW_DEVICE_ID_HEADSET_MONO_PLUS_SPKR_STEREO_RX) ||
							(dev_id == CAD_HW_DEVICE_ID_HEADSET_STEREO_PLUS_SPKR_STEREO_RX))
						{
							gpio_set_value(AUDIO_HPSHDN_SHTDWN_N, 0);
							
							PM_LOG_EVENT(PM_LOG_OFF, PM_LOG_AUDIO_HS);
						}
						
					}
					else
					{
						pmic_speaker_cmd(SPKR_MUTE_ON);
					}
				}
			}
			else if ((dev_id == CAD_HW_DEVICE_ID_HEADSET_SPKR_MONO) ||
					(dev_id == CAD_HW_DEVICE_ID_HEADSET_SPKR_STEREO) ||
					(dev_id == CAD_HW_DEVICE_ID_HEADSET_SPKR_STEREO_LB) ||
					(dev_id == CAD_HW_DEVICE_ID_HEADSET_SPKR_MONO_LB) ||
					(dev_id == CAD_HW_DEVICE_ID_TTY_HEADSET_SPKR))
			{
				gpio_set_value(AUDIO_HPSHDN_SHTDWN_N, 0);
				
				PM_LOG_EVENT(PM_LOG_OFF, PM_LOG_AUDIO_HS);

			}
			else
			{
				pr_err("error: not stereo or mono\n");
			}
		}
		else
		{
			D("ARD ADIE Loopback Device\n");
		}

		
		dal_rc = rpc_adie_codec_proceed_to_stage(adie_state.adie_handle,
				path_type, ADIE_CODEC_ANALOG_OFF);

		if (dal_rc != CAD_RES_SUCCESS) {
			pr_err("ARD ADIE Proceed to Stage failed,"
				" dev_type %d\n", dev_type);
			rc = ADIE_STATE_RC_FAILURE;
			goto done;
		}

		adie_state.adie_path_type[dev_type].state =
			ADIE_STATE_DIGITAL_ACTIVE;
		adie_state.adie_path_type[dev_type].enable_request = ADIE_FALSE;
		rc = ADIE_STATE_RC_CONTINUE;
	}
done:
	return rc;
}

