/* Copyright (c) 2009, Code Aurora Forum. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Code Aurora Forum nor
 *       the names of its contributors may be used to endorse or promote
 *       products derived from this software without specific prior written
 *       permission.
 *
 * Alternatively, provided that this notice is retained in full, this software
 * may be relicensed by the recipient under the terms of the GNU General Public
 * License version 2 ("GPL") and only version 2, in which case the provisions of
 * the GPL apply INSTEAD OF those given above.  If the recipient relicenses the
 * software under the GPL, then the identification text in the MODULE_LICENSE
 * macro must be changed to reflect "GPLv2" instead of "Dual BSD/GPL".  Once a
 * recipient changes the license terms to the GPL, subsequent recipients shall
 * not relicense under alternate licensing terms, including the BSD or dual
 * BSD/GPL terms.  In addition, the following license statement immediately
 * below and between the words START and END shall also then apply when this
 * software is relicensed under the GPL:
 *
 * START
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License version 2 and only version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * END
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */




#include <linux/kernel.h>
#include <linux/kthread.h>

#include <mach/msm_rpcrouter.h>
#include "smd_rpcrouter.h"

struct msm_rpc_client_cb_item {
	struct list_head list;

	void *buf;
	int size;
};

struct msm_rpc_cb_table_item {
	struct list_head list;

	uint32_t cb_id;
	void *cb_func;
};

#ifdef CONFIG_SPEEDUP_KEYRESUME
#define HS_RPC_CB_PROG 0x31000091
#endif
static int rpc_clients_cb_thread(void *data)
{
	struct msm_rpc_client_cb_item *cb_item;
	struct msm_rpc_client *client;
	struct rpc_request_hdr *req;

	client = data;
	for (;;) {
		wait_event(client->cb_wait, client->cb_avail);
		if (client->exit_flag)
			break;

		client->cb_avail = 0;
		mutex_lock(&client->cb_item_list_lock);
		while (!list_empty(&client->cb_item_list)) {
			cb_item = list_first_entry(
				&client->cb_item_list,
				struct msm_rpc_client_cb_item,
				list);
			list_del(&cb_item->list);
			mutex_unlock(&client->cb_item_list_lock);
			req = (struct rpc_request_hdr *)cb_item->buf;

			if (be32_to_cpu(req->type) != 0)
				goto bad_rpc;
			if (be32_to_cpu(req->rpc_vers) != 2)
				goto bad_rpc;
			if (be32_to_cpu(req->prog) !=
			    (client->prog | 0x01000000))
				goto bad_rpc;

			client->cb_func(client,
					cb_item->buf, cb_item->size);
 bad_rpc:
			kfree(cb_item->buf);
			kfree(cb_item);
			mutex_lock(&client->cb_item_list_lock);
		}
		mutex_unlock(&client->cb_item_list_lock);
	}
	complete_and_exit(&client->cb_complete, 0);
}

static int rpc_clients_thread(void *data)
{
	void *buffer;
	uint32_t type;
	struct msm_rpc_client *client;
	int rc = 0;
	struct msm_rpc_client_cb_item *cb_item;
	struct rpc_request_hdr *req;

#ifdef CONFIG_SPEEDUP_KEYRESUME
	struct sched_param smdrpcrouterclient_s = { .sched_priority = 88 };
	struct sched_param smdrpcrouterclient_v = { .sched_priority = 0 };
	int smdrpcrouterclient_old_prio = 0;
 	int smdrpcrouterclient_old_policy = 0;
#endif
	client = data;
	for (;;) {
		rc = msm_rpc_read(client->ept, &buffer, -1, HZ);
		if (client->exit_flag)
			break;
		if (rc < ((int)(sizeof(uint32_t) * 2)))
			continue;

		type = be32_to_cpu(*((uint32_t *)buffer + 1));
		if (type == 1) {
			client->buf = buffer;
			client->read_avail = 1;
			wake_up(&client->reply_wait);
		} else if (type == 0) {
			cb_item = kmalloc(sizeof(*cb_item), GFP_KERNEL);
			if (!cb_item) {
				pr_err("%s: no memory for cb item\n",
				       __func__);
				continue;
			}

			if (client->cb_thread == NULL) {
				req = (struct rpc_request_hdr *)buffer;

				if ((be32_to_cpu(req->rpc_vers) == 2) &&
				    (be32_to_cpu(req->prog) ==
				     (client->prog | 0x01000000)))
				{
#ifdef CONFIG_SPEEDUP_KEYRESUME
					if ( be32_to_cpu(req->prog)== HS_RPC_CB_PROG)
					{
						
						
						smdrpcrouterclient_old_prio = current->rt_priority;
 						smdrpcrouterclient_old_policy = current->policy;
 						if (!(unlikely(smdrpcrouterclient_old_policy == SCHED_FIFO) || unlikely(smdrpcrouterclient_old_policy == SCHED_RR)))
 						{
							if ((sched_setscheduler(current, SCHED_FIFO, &smdrpcrouterclient_s)) < 0)
   								printk(KERN_ERR "[SEAN]uppp rpc_clients_thread fail\n");
   							
   							
   						}
   						client->cb_func(client, buffer, rc);
 						
 						if (!(unlikely(smdrpcrouterclient_old_policy == SCHED_FIFO) || unlikely(smdrpcrouterclient_old_policy == SCHED_RR)))
 						{
  							smdrpcrouterclient_v.sched_priority = smdrpcrouterclient_old_prio;
  							if ((sched_setscheduler(current, smdrpcrouterclient_old_policy, &smdrpcrouterclient_v)) < 0)
   								printk(KERN_ERR "[SEAN]down rpc_clients_thread failed\n");
   							
   							
  						}
					}
					else
#endif
					client->cb_func(client, buffer, rc);
				}
				kfree(buffer);
                kfree(cb_item);
			} else {
				INIT_LIST_HEAD(&cb_item->list);
				cb_item->buf = buffer;
				cb_item->size = rc;
				mutex_lock(&client->cb_item_list_lock);
				list_add_tail(&cb_item->list,
					      &client->cb_item_list);
				mutex_unlock(&client->cb_item_list_lock);
				client->cb_avail = 1;
				wake_up(&client->cb_wait);
			}
		}
	}
	complete_and_exit(&client->complete, 0);
}

static struct msm_rpc_client *msm_rpc_create_client(void)
{
	struct msm_rpc_client *client;

	client = kmalloc(sizeof(struct msm_rpc_client), GFP_KERNEL);
	if (!client)
		return ERR_PTR(-ENOMEM);

	client->req = kmalloc(MSM_RPC_MSGSIZE_MAX, GFP_KERNEL);
	if (!client->req) {
		kfree(client);
		return ERR_PTR(-ENOMEM);
	}

	client->reply = kmalloc(MSM_RPC_MSGSIZE_MAX, GFP_KERNEL);
	if (!client->reply) {
		kfree(client->req);
		kfree(client);
		return ERR_PTR(-ENOMEM);
	}

	init_waitqueue_head(&client->reply_wait);
	mutex_init(&client->req_lock);
	mutex_init(&client->reply_lock);
	client->buf = NULL;
	client->read_avail = 0;
	client->cb_buf = NULL;
	client->cb_size = 0;
	client->exit_flag = 0;
	init_completion(&client->complete);
	init_completion(&client->cb_complete);
	INIT_LIST_HEAD(&client->cb_item_list);
	mutex_init(&client->cb_item_list_lock);
	client->cb_avail = 0;
	init_waitqueue_head(&client->cb_wait);
	INIT_LIST_HEAD(&client->cb_list);
	mutex_init(&client->cb_list_lock);
	atomic_set(&client->next_cb_id, 1);

	return client;
}

void msm_rpc_remove_all_cb_func(struct msm_rpc_client *client)
{
	struct msm_rpc_cb_table_item *cb_item, *tmp_cb_item;

	mutex_lock(&client->cb_list_lock);
	list_for_each_entry_safe(cb_item, tmp_cb_item,
				 &client->cb_list, list) {
		list_del(&cb_item->list);
		kfree(cb_item);
	}
	mutex_unlock(&client->cb_list_lock);
}


struct msm_rpc_client *msm_rpc_register_client(
	const char *name,
	uint32_t prog, uint32_t ver,
	uint32_t create_cb_thread,
	int (*cb_func)(struct msm_rpc_client *, void *, int))
{
	struct msm_rpc_client *client;
	struct msm_rpc_endpoint *ept;
	int rc;

	client = msm_rpc_create_client();
	if (IS_ERR(client))
		return client;

	ept = msm_rpc_connect_compatible(prog, ver, MSM_RPC_UNINTERRUPTIBLE);
	if (IS_ERR(ept)) {
		kfree(client);
		return (struct msm_rpc_client *)ept;
	}

	client->prog = prog;
	client->ver = ver;
	client->ept = ept;
	client->cb_func = cb_func;

	
	client->read_thread = kthread_run(rpc_clients_thread, client,
					  "k%sclntd", name);
	if (IS_ERR(client->read_thread)) {
		rc = PTR_ERR(client->read_thread);
		msm_rpc_close(client->ept);
		kfree(client);
		return ERR_PTR(rc);
	}

	if (!create_cb_thread || (cb_func == NULL)) {
		client->cb_thread = NULL;
		return client;
	}

	
	client->cb_thread = kthread_run(rpc_clients_cb_thread, client,
					"k%sclntcbd", name);
	if (IS_ERR(client->cb_thread)) {
		rc = PTR_ERR(client->cb_thread);
		client->exit_flag = 1;
		wait_for_completion(&client->complete);
		msm_rpc_close(client->ept);
		kfree(client);
		return ERR_PTR(rc);
	}

	return client;
}
EXPORT_SYMBOL(msm_rpc_register_client);


int msm_rpc_unregister_client(struct msm_rpc_client *client)
{
	pr_info("%s: stopping client...\n", __func__);
	client->exit_flag = 1;
	if (client->cb_thread) {
		client->cb_avail = 1;
		wake_up(&client->cb_wait);
		wait_for_completion(&client->cb_complete);
	}

	wait_for_completion(&client->complete);

	msm_rpc_close(client->ept);
	msm_rpc_remove_all_cb_func(client);
	kfree(client->req);
	kfree(client->reply);
	kfree(client);
	return 0;
}
EXPORT_SYMBOL(msm_rpc_unregister_client);


int msm_rpc_client_req(struct msm_rpc_client *client, uint32_t proc,
		       int (*arg_func)(struct msm_rpc_client *client,
				       void *buf, void *data),
		       void *arg_data,
		       int (*ret_func)(struct msm_rpc_client *client,
				       void *buf, void *data),
		       void *ret_data, long timeout)
{
	int size = 0;
	struct rpc_reply_hdr *rpc_rsp;
	int rc = 0;

	mutex_lock(&client->req_lock);

	msm_rpc_setup_req((struct rpc_request_hdr *)client->req, client->prog,
			  client->ver, proc);
	size = sizeof(struct rpc_request_hdr);

	if (arg_func) {
		rc = arg_func(client, (void *)((struct rpc_request_hdr *)
					       client->req + 1), arg_data);
		if (rc < 0)
			goto release_locks;
		else
			size += rc;
	}

	rc = msm_rpc_write(client->ept, client->req, size);
	if (rc < 0) {
		pr_err("%s: couldn't send RPC request:%d\n", __func__, rc);
		goto release_locks;
	} else
		rc = 0;

	if (timeout < 0)
		timeout = msecs_to_jiffies(10000);

	rc = wait_event_timeout(client->reply_wait,
				client->read_avail, timeout);
	if (rc == 0) {
		rc = -ETIMEDOUT;
		goto release_locks;
	} else
		rc = 0;

	client->read_avail = 0;

	rpc_rsp = (struct rpc_reply_hdr *)client->buf;
	if (be32_to_cpu(rpc_rsp->reply_stat) != RPCMSG_REPLYSTAT_ACCEPTED) {
		pr_err("%s: RPC call was denied! %d\n", __func__,
		       be32_to_cpu(rpc_rsp->reply_stat));
		rc = -EPERM;
		goto free_and_release;
	}

	if (be32_to_cpu(rpc_rsp->data.acc_hdr.accept_stat) !=
	    RPC_ACCEPTSTAT_SUCCESS) {
		pr_err("%s: RPC call was not successful (%d)\n", __func__,
		       be32_to_cpu(rpc_rsp->data.acc_hdr.accept_stat));
		rc = -EINVAL;
		goto free_and_release;
	}

	if (ret_func)
		rc = ret_func(client, (void *)(rpc_rsp + 1), ret_data);

 free_and_release:
	kfree(client->buf);
 release_locks:
	mutex_unlock(&client->req_lock);
	return rc;
}
EXPORT_SYMBOL(msm_rpc_client_req);


void *msm_rpc_start_accepted_reply(struct msm_rpc_client *client,
				   uint32_t xid, uint32_t accept_status)
{
	struct rpc_reply_hdr *reply;

	mutex_lock(&client->reply_lock);

	reply = (struct rpc_reply_hdr *)client->reply;

	reply->xid = cpu_to_be32(xid);
	reply->type = cpu_to_be32(1); 
	reply->reply_stat = cpu_to_be32(RPCMSG_REPLYSTAT_ACCEPTED);

	reply->data.acc_hdr.accept_stat = cpu_to_be32(accept_status);
	reply->data.acc_hdr.verf_flavor = 0;
	reply->data.acc_hdr.verf_length = 0;

	return reply + 1;
}
EXPORT_SYMBOL(msm_rpc_start_accepted_reply);


int msm_rpc_send_accepted_reply(struct msm_rpc_client *client, uint32_t size)
{
	int rc = 0;

	size += sizeof(struct rpc_reply_hdr);
	rc = msm_rpc_write(client->ept, client->reply, size);
	if (rc > 0)
		rc = 0;

	mutex_unlock(&client->reply_lock);
	return rc;
}
EXPORT_SYMBOL(msm_rpc_send_accepted_reply);


int msm_rpc_add_cb_func(struct msm_rpc_client *client, void *cb_func)
{
	struct msm_rpc_cb_table_item *cb_item;

	if (cb_func == NULL)
		return MSM_RPC_CLIENT_NULL_CB_ID;

	mutex_lock(&client->cb_list_lock);
	list_for_each_entry(cb_item, &client->cb_list, list) {
		if (cb_item->cb_func == cb_func) {
			mutex_unlock(&client->cb_list_lock);
			return cb_item->cb_id;
		}
	}
	mutex_unlock(&client->cb_list_lock);

	cb_item = kmalloc(sizeof(struct msm_rpc_cb_table_item), GFP_KERNEL);
	if (!cb_item)
		return -ENOMEM;

	INIT_LIST_HEAD(&cb_item->list);
	cb_item->cb_id = atomic_add_return(1, &client->next_cb_id);
	cb_item->cb_func = cb_func;

	mutex_lock(&client->cb_list_lock);
	list_add_tail(&cb_item->list, &client->cb_list);
	mutex_unlock(&client->cb_list_lock);

	return cb_item->cb_id;
}
EXPORT_SYMBOL(msm_rpc_add_cb_func);


void *msm_rpc_get_cb_func(struct msm_rpc_client *client, uint32_t cb_id)
{
	struct msm_rpc_cb_table_item *cb_item;

	mutex_lock(&client->cb_list_lock);
	list_for_each_entry(cb_item, &client->cb_list, list) {
		if (cb_item->cb_id == cb_id) {
			mutex_unlock(&client->cb_list_lock);
			return cb_item->cb_func;
		}
	}
	mutex_unlock(&client->cb_list_lock);
	return NULL;
}
EXPORT_SYMBOL(msm_rpc_get_cb_func);


void msm_rpc_remove_cb_func(struct msm_rpc_client *client, void *cb_func)
{
	struct msm_rpc_cb_table_item *cb_item, *tmp_cb_item;

	if (cb_func == NULL)
		return;

	mutex_lock(&client->cb_list_lock);
	list_for_each_entry_safe(cb_item, tmp_cb_item,
				 &client->cb_list, list) {
		if (cb_item->cb_func == cb_func) {
			list_del(&cb_item->list);
			kfree(cb_item);
			mutex_unlock(&client->cb_list_lock);
			return;
		}
	}
	mutex_unlock(&client->cb_list_lock);
}
EXPORT_SYMBOL(msm_rpc_remove_cb_func);
